@@ -20,6 +20,7 @@ use std::cmp::Ordering;
 use std::fmt;
 use std::iter::{Product, Sum};
 use std::ops::*;
+use std::str::FromStr;
 
 use async_trait::async_trait;
 use collate::*;
@@ -37,6 +38,28 @@ pub use class::*;
 use destream::Encoder;
 pub use instance::*;
 
+pub struct Error(String);
+
+impl Error {
+    fn new<E: fmt::Display>(cause: E) -> Self {
+        Self(cause.to_string())
+    }
+}
+
+impl std::error::Error for Error {}
+
+impl fmt::Debug for Error {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.write_str(&self.0)
+    }
+}
+
+impl fmt::Display for Error {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.write_str(&self.0)
+    }
+}
+
 type _Complex<T> = num::complex::Complex<T>;
 
 const EXPECTING: &str = "a Number, like 1 or -2 or 3.14 or [0., -1.414]";
@@ -52,6 +75,7 @@ pub enum Number {
 }
 
 impl Number {
+    /// Return `false` if this is a [`Complex`] `Number`.
     pub fn is_real(&self) -> bool {
         if let Self::Complex(_) = self {
             false
@@ -135,6 +159,24 @@ impl NumberInstance for Number {
     }
 }
 
+impl FloatInstance for Number {
+    fn is_infinite(&self) -> bool {
+        match self {
+            Self::Complex(c) => c.is_infinite(),
+            Self::Float(f) => f.is_infinite(),
+            _ => false,
+        }
+    }
+
+    fn is_nan(&self) -> bool {
+        match self {
+            Self::Complex(c) => c.is_nan(),
+            Self::Float(f) => f.is_nan(),
+            _ => false,
+        }
+    }
+}
+
 impl PartialEq for Number {
     fn eq(&self, other: &Self) -> bool {
         match (self, other) {
@@ -478,6 +520,28 @@ impl From<Complex> for Number {
     }
 }
 
+impl FromStr for Number {
+    type Err = Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let s = s.trim();
+
+        if s.is_empty() {
+            Err(Error(
+                "cannot parse a Number from an empty string".to_string(),
+            ))
+        } else if s.len() == 1 {
+            Int::from_str(s).map(Self::from)
+        } else if s[1..].contains('+') || s[1..].contains('-') {
+            Complex::from_str(s).map(Self::from)
+        } else if s.contains('.') {
+            Float::from_str(s).map(Self::from)
+        } else {
+            Int::from_str(s).map(Self::from)
+        }
+    }
+}
+
 impl CastFrom<Number> for Boolean {
     fn cast_from(number: Number) -> Boolean {
         if number == number.class().zero() {
@@ -757,6 +821,11 @@ impl<'de> serde::de::Visitor<'de> for NumberVisitor {
         self.f64(f)
     }
 
+    #[inline]
+    fn visit_str<E: SerdeError>(self, s: &str) -> Result<Self::Value, E> {
+        s.parse().map_err(serde::de::Error::custom)
+    }
+
     #[inline]
     fn visit_seq<A: serde::de::SeqAccess<'de>>(self, mut seq: A) -> Result<Self::Value, A::Error> {
         let re = seq
@@ -834,6 +903,11 @@ impl destream::de::Visitor for NumberVisitor {
         self.f64(f)
     }
 
+    #[inline]
+    fn visit_string<E: DestreamError>(self, s: String) -> Result<Self::Value, E> {
+        s.parse().map_err(destream::de::Error::custom)
+    }
+
     async fn visit_seq<A: destream::de::SeqAccess>(
         self,
         mut seq: A,
@@ -1044,4 +1118,23 @@ mod tests {
 
         assert_eq!(deserialized, numbers);
     }
+
+    #[test]
+    fn test_parse() {
+        assert_eq!(Number::from_str("12").unwrap(), Number::from(12));
+        assert_eq!(Number::from_str("+31.4").unwrap(), Number::from(31.4));
+        assert_eq!(Number::from_str("-3.14").unwrap(), Number::from(-3.14));
+        assert_eq!(
+            Number::from_str("3.14+1.414i").unwrap(),
+            Number::from(num::Complex::new(3.14, 1.414))
+        );
+        assert_eq!(
+            Number::from_str("1+2i").unwrap(),
+            Number::from(num::Complex::new(1., 2.))
+        );
+        assert_eq!(
+            Number::from_str("1.0-2i").unwrap(),
+            Number::from(num::Complex::new(1., -2.))
+        );
+    }
 }
